/****************************************************************************************
*                   Copyright (c) Gemplus Card International 1996                       
*                                                                                       
* Name        : MPCPURSE.C                                                               
*                                                                                       
* Description : Sample for 3-DES MPCOS Library.
*               This program performs 3DES - MPCOS standard purse functions.
*                                                                                       
* Author      : PP & CB                                                                      
*                                                                                       
* Compiler    : Microsoft Visual C++ / QuickWin                                         
*               Borland C++ / EasyWin                                                   
*                                                                                       
* Host        : IBM PC and compatible machines under Windows 3.1                                                             
*                                                                                       
* Release     : 1.1                                                                     
*                                                                                       
* Date        : 
* Last Modif. : May 96                                                                                                                                        
*                                                                                       
*****************************************************************************************
*                                                                                       
* Warning     : THIS SAMPLE PROGRAM IS NOT MEANT TO SHOW HOW TO DEVELOP REAL            
*               SMART CARDS APPLICATIONS.                                               
*               IT IS ONLY WRITTEN TO SHOW DEVELOPERS HOW TO ACCESS SMART               
*               CARDS THROUGH THE GCR 'C' LIBRARY AND MPCOS 'C' LIBRARY.                
*                                                                                       
*                                                                                       
*****************************************************************************************/


/*--------------------------------------------------------------------------------------
                             Information section
----------------------------------------------------------------------------------------*/

#define G_NAME     "MPCPURSE"
#define G_RELEASE  "1.1"

/*--------------------------------------------------------------------------------------
                                Include section
----------------------------------------------------------------------------------------*/

#define STRICT
#include <windows.h>
#include<windef.h>
#include <stdio.h>
#include <stdlib.h> 
#include <string.h>
#define G_WINDOWS
#include "gemplus.h"             /* For Gemplus general declarations             */
#include "wscump2.h"             /* Smart Card Utilities for MPCOS cards         */


/*--------------------------------------------------------------------------------------
                                Constant section
----------------------------------------------------------------------------------------*/
/* Reader parameters                                                                    */
#define IFD_TYPE  GCR400        /* GCR400    type                                       */
#define IFD_MODE  G_SERIAL      /* serial communication                                 */
#define IFD_PORT  G_COM1        /* Reader Connected on COM 1                            */
#define IFD_BAUD  9600          /* Speed                                                */
#define IFD_IT    0xFF          /* Interrupt                                            */

                                 
#define SOFT_COMPUTE -1         /* This value , used in some MPCOS functions,           */
                                /* indicates that the calculation is made by            */
                                /* the terminal and not in the security module          */ 
 
#define KEY_FILE_ID  0x3F01     /* System key file identifier                           */
#define KEY_NUMBER   0          /* Number of the system key                             */
#define KEY_REF      "TEST KEYTEST KEY" /* Value used to set the session key                    */      
                                
/*--------------------------------------------------------------------------------------
                             Function definition section
----------------------------------------------------------------------------------------*/
 
 
/****************************************************************************************
* INT16 OpenChannel(INT16 IFDType)
*
* Description : This function opens the communication with the card reader.
*
* Remarks     : Nothing.
*
* In          : IFDType: Type of card reader.
*
* Out         : Nothing.
*
* Response    : The open channel identifier.

*****************************************************************************************/
INT16 OpenChannel(INT16 IFDType){
   G4_CHANNEL_PARAM
      channel;
   INT16
      ChannelID;

   /* prepares channel parameters                                                       */
   channel.IFDType              = IFDType; 
   channel.IFDMode              = IFD_MODE;
   channel.Comm.Serial.Port     = IFD_PORT; 
   channel.Comm.Serial.BaudRate = IFD_BAUD;
   channel.Comm.Serial.ITNumber = IFD_IT;
      

   /* opens communication with the reader                                               */
   printf("* Open Channel: ");
   ChannelID = G4_OpenChannel(&channel);
   if (ChannelID < G_OK)
   {
      printf(" Failure.    G4_OpenChannel returns %d.\n",ChannelID);
   }
   else
   {
      printf("OK.\n");
   }

   return ChannelID;
}
/****************************************************************************************
* void  CloseChannel(INT16 ChannelID)
*
* Description : This function closes the communication with a card reader
*
* Remarks     : Nothing.
*
* In          : ChannelID: The Channel identifier.
*
* Out         : Nothing.

*****************************************************************************************/
void CloseChannel(INT16 ChannelID)
{
   INT16
      response;

   printf("* Close Channel: ");
   
   response = G4_CloseChannel(ChannelID);
   
   if (response < G_OK){
      printf(" Failure  ");
      printf("G4_CloseChannel returns %d.\n",response);
   }
   else{
      printf("OK.\n");
   }
}

/****************************************************************************************
* void  OpenSession  (INT16 ChannelID)
*
* Description : This function opens a session with a MPCOS card, Initializes the 
*               library and sets the value of the IO buffer size according to the
*               type of the card.
*
* Remarks     : Nothing.
*
* In          : ChannelID: The channel identifier.
*
* Out         : Nothing.
*
*****************************************************************************************/
void OpenSession(INT16 ChannelID)
{
   INT16
      response;
   G4_SESSION_PARAM 
      session; 
   WORD8 
      DFDescriptor[18],
      szIssuingData[12],
      ComponentReference, 
      CodeReference, 
      Version;             
   WORD16 
      nCardType; 


   /* Open session --------------------------------------------------------------------*/
  
   session.ICCType = ISOCARD ;
  
   printf("* Open Session: ");
   
   response = G4_OpenSession(ChannelID,
                             &session);
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G4_OpenSession returns %d.\n",response);
   }
   else{
      printf("OK.\n");   
   }

   /* Init library ---------------------------------------------------------------------*/
   
   printf("* Init Library: ");
   response = G_MPC_InitLib (ChannelID,
                             DFDescriptor);
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_InitLib returns %d.\n",response);
   }
   else{
      printf("OK.\n");   
   }                 

  /* Get Info --------------------------------------------------------------------------*/
  
   printf("* Get Info: ");
   response = G_MPC_GetInfo (ChannelID,
                             0xA4,          /* Code to recover the chip type infos      */
                             12,            /* length expected                          */
                             szIssuingData);
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_GetInfo returns %d.\n",response);
   }
   else{
      printf("OK.\n");   
   }                 
   
   ComponentReference = szIssuingData[0];
   CodeReference      = szIssuingData[1];
   Version            = szIssuingData[2];
   
   
  /* Set IO size -----------------------------------------------------------------------*/    
  
      nCardType = UNKNOWN_CHIP;
   
   /* These are the common component references and versions of MPCOS cards             */
   
      if (CodeReference == 0xA0){
      if ((ComponentReference == 0x25) && (Version == 0x52))   nCardType = MPCOS24K;
      if ((ComponentReference == 0x30) && (Version == 0x52))   nCardType = MPCOS64K;
      if ((ComponentReference == 0x30) && (Version == 0x55))   nCardType = MPCOS64K;
      if ((ComponentReference == 0x31) && (Version == 0x5A))   nCardType = MPCOS64K;
      if ((ComponentReference == 0x32) && (Version == 0x54))   nCardType = MPCOS8K;
      if ((ComponentReference == 0x3B) && (Version == 0x56))   nCardType = MPCOS32K;
      if ((ComponentReference == 0x3C) && (Version == 0x59))   nCardType = MPCOS32K;
   }
                               
   switch(nCardType){  
   
    case  MPCOS8K : G_MPC_SetIOSize(16); /* Buffer size is 16 for MPCOS8K               */
                    break;
    case  MPCOS24K: G_MPC_SetIOSize(32); /* Buffer size is 32 for MPCOS24K              */
                    break;
    case  MPCOS32K: G_MPC_SetIOSize(96); /* Buffer size is 96 for MPCOS32K              */
                    break;
    case  MPCOS64K: G_MPC_SetIOSize(96); /* Buffer size is 96 for MPCOS64K              */
                    break;                                                           
   /* If the card is not in the list the default value of the IO buffer size is 16      */
    default       : G_MPC_SetIOSize(16); 
                    break;                                                           
   }            
                    

}

/****************************************************************************************
* void  CloseSession(INT16 ChannelID)
*
* Description : This function closes the session with a card
*
* Remarks     : Nothing.
*
* In          : ChannelID: The Channel identifier.
*
* Out         : Nothing.

*****************************************************************************************/
void CloseSession(INT16 ChannelID)
{
   INT16 
      response;
   
   printf("* Close Session: ");
   
   response = G4_CloseSession(ChannelID);
   
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G4_CloseSession returns %d.\n",ChannelID);
   }
   else{
      printf("OK.\n");
   }
 
}           

/****************************************************************************************
* void  SelectSystemKey(INT16 ChannelId,WORD16 SystemKeyEFIdentifier)
*
* Description : This function select a key in the system key file, establish a session
*               key, allows the terminal to authenticate the card and to compute 
*               internally the session key for secure messaging commands
*
* Remarks     : Nothing.
*
* In          : ChannelId:             The channel identifier
*               SystemKeyEFIdentifier  The system key file
*
* Out         : Nothing.
*
*****************************************************************************************/
void SelectSystemKey(INT16 ChannelId,WORD16 SystemKeyEFIdentifier)
{
   INT16 
      response;
   WORD8  
      szTerminalRandom[8],        /* Random string build by the terminal                */
      szCardRandom[8],            /* Random string sent by the card                     */
      szResult[4];                /* 4 least significant bytes of the cryptogram        */
                                  /* returned by the card                               */  
   int i;


   for (i=0;i<8;i++)
   szTerminalRandom[i] = (unsigned char) rand();
  

   printf("* Select System Key: ");
                                                    
   response = G_MPC_SelKey(ChannelId,
                           MF,               
                           SystemKeyEFIdentifier,
                           KEY_NUMBER,
                           szTerminalRandom,
                           szResult,
                           szCardRandom);
   
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_SelKey returns %d.\n",response);
   }
   else{
      response = G_MPCSM_SetSecureSessionKey(SOFT_COMPUTE,
                                            MF,               
                                            SystemKeyEFIdentifier,
                                            KEY_NUMBER,
                                             szTerminalRandom,
                                             szCardRandom,
                                             szResult,
                                             KEY_REF);
   
      if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPCSM_SetSecureSessionKey returns %d.\n",response);
      }
      else{
           printf("OK\n");
     }
   }
      
}

/****************************************************************************************
* void CreateDF(INT16 ChannelId,WORD16 DFIdentifier,BYTE G_FAR *DFName)
*
* Description : This function creates a DF with the 3-DES crypto options
*
* Remarks     : Nothing.
*
* In          : ChannelID:     The channel identifier
*               DFIdentifier:  The identifier of the DF to create
*               DFName:        The name of the DF to create.
*
* Out         : Nothing.
*
*****************************************************************************************/
void CreateDF(INT16 ChannelId,WORD16 DFIdentifier,BYTE G_FAR *DFName)
{
   ACC    
      SC_Group[3];
   BYTE  
      SC_MAC_In[3],     /* Buffer to recover the certificate passed by the command      */
      SC_MAC_Out[3];    /* Buffer to recover the certificate returned by the command    */
   INT16   
      response,
      GeneralOptions,
      OnLineOptions,
      CryptoOptions;
   int i;
  
/*    Initialization of the options                                                     */
   GeneralOptions = ISO_DF           /* The entity to create is ISO                     */
                  + SUBDBT           /* The substitute debit command is allowed         */
                  + NO_CUR_BAL_SIGN; /* The current balance can't be used for computing */
                                     /* the sign certificate.                           */ 

   OnLineOptions = 0;                /* Unblock code, Update binary, Read binary, create*/
                                     /* and freeze and credit are not used like on line */
                                     /* commands                                        */
   
   CryptoOptions = FULL_TRIPLE_DES;  /* Cryptographic method                            */
                                     /*     3-DES for Session keys computation          */
                                     /*     3-DES for Authentication, Certificate       */
                                     /*           and Signatures computation            */
   
        
/*    Initialization of the access conditions                                           */
   for (i=0;i<3;i++){                /* No access conditions                            */
     SC_Group[i].Validity=0;
     SC_Group[i].KeyLevel=0;
     SC_Group[i].KeyShortID=0;
     SC_Group[i].Sc1Level=0;
     SC_Group[i].Sc2Level=0;
     SC_Group[i].Sc1Number=0;
     SC_Group[i].Sc2Number=0;
   }     
        
   printf("* Create DF: ");
   
   response = G_MPC_CreateDFSecure(ChannelId,
                                   SOFT_COMPUTE,
                                   DFIdentifier,
                                   GeneralOptions,
                                   OnLineOptions,
                                   CryptoOptions,
                                   strlen(DFName),
                                   &(SC_Group[0]),
                                   &(SC_Group[1]),
                                   &(SC_Group[2]),
                                   DFName,
                                   SC_MAC_In,
                                   SC_MAC_Out);
                                  
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_CreateDFSecure returns %d.\n",response);
   }
   else{
      printf("OK.\n");
   }
}
/****************************************************************************************
* void  SelectDF(INT16 ChannelID,WORD16 DFIdentifier)
*
* Description : This function selects a DF using its identifier.
*
* Remarks     : Nothing.
*
* In          : ChannelID: The Channel identifier
*               DFIdentifier: The DF identifier.
*
* Out         : Nothing.

*****************************************************************************************/
void SelectDF(INT16 ChannelID, WORD16 DFIdentifier)    
{
   WORD8 
      Descriptor[36];
   INT16 
      Mode,
      response;                        
   
   Mode = 1;              /* The mode is 1 to select a DF file                          */
    
   printf("* Select DF: ");
    
   response = G_MPC_Select(ChannelID, 
                           Mode, 
                           DFIdentifier, 
                           Descriptor);                          
  
            
   /* Descriptor contains  "85" +                                                       */
   /*                       "10"(size of the DF descriptor in hexa) +                   */
   /*                        the 16 bytes of the DF descriptor +                        */
   /*                        "84" +                                                     */
   /*                        the length of the DF name +                                */
   /*                        the DF name (up to 16 bytes).                              */
                                                                                      
   
   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_Select returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
     
}
/****************************************************************************************
* void CreateKeyEF(INT16 ChannelID,WORD16 KeyEFIdentifier)
*
* Description : This function creates an EF
*
* Remarks     : Nothing.
*
* In          : ChannelID:       The channel identifier
*               KeyEFIdentifier: The identifier of the EF to create
*
* Out         : Nothing.

*****************************************************************************************/
void CreateKeyEF(INT16 ChannelID,WORD16 KeyEFIdentifier)
{
   INT16 
      response,
      EF_Size;
   ACC   
      SC_Group[3];  
   int i;
                 
/*    Initialization of the variables                                                   */

   EF_Size = 48;    /* Size of the EF in bytes                                           */         
   
        
/*    Initialization of the access conditions                                           */
 
   for (i=0;i<3;i++){                /* No access conditions                            */
     SC_Group[i].Validity=0;
     SC_Group[i].KeyLevel=0;
     SC_Group[i].KeyShortID=0;
     SC_Group[i].Sc1Level=0;
     SC_Group[i].Sc2Level=0;
     SC_Group[i].Sc1Number=0;
     SC_Group[i].Sc2Number=0;
   }     
                 
   printf("* Create key EF: ");

   response = G_MPC_CreateEF(ChannelID,
                             KeyEFIdentifier, 
                             EF_KEY, 
                             EF_Size, 
                             &SC_Group[0], 
                             &SC_Group[1], 
                             &SC_Group[2]);
    

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_CreateEF returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
}
/****************************************************************************************
* void LoadPayKey(INT16 ChannelID,WORD16 KeyEFIdentifier,BYTE G_FAR *KeyValue)
*
* Description : This function loads two simple payment keys in the key file.
*
* Remarks     : Nothing.
*
* In          : ChannelID:       The channel identifier.
*               KeyEFIdentifier: The key file identifier.
*               KeyValue:        The value of the key (2 x 8 bytes)
*
*
* Out         : Nothing.

*****************************************************************************************/
void LoadPayKey(INT16 ChannelID,WORD16 KeyEFIdentifier,BYTE G_FAR *KeyValue)
{
   INT16 
      response,  
      KeyNumber,                                               
      Mode,
      KeyType;
    
/*    Initialization of the variables                                                   */

   Mode       = SELECT;     /* Using with select file                                   */
   KeyNumber  = 0;          /* Number of the key in the key file                        */ 
   KeyType    = PAY;        /* Payment key                                              */  
 
    
   printf("* Load payment keys: ");
    
   response = G_MPC_LoadKey(ChannelID,
                            Mode,
                            KeyEFIdentifier,
                            KeyNumber,
                            KeyType,  
                            0,
                            KeyValue                            
                            );
   
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_LoadKey returns %d.\n",response);
   }
   else{
   
      Mode       = NO_SELECT;  /* File already selected                                    */
      KeyNumber  = 1;          /* Number of the key in the key file                        */ 
      KeyType    = PAY;        /* Payment key                                              */  
                 
      response = G_MPC_LoadKey(ChannelID,
                               Mode,
                               KeyEFIdentifier,
                               KeyNumber,
                               KeyType,  
                               0,
                               KeyValue +8                           
                               );
      
      if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_LoadKey returns %d.\n",response);
      }
      else{
      
         printf("OK.\n");
      }
   }
}
/****************************************************************************************
* void LoadCreditKey(INT16 ChannelID,BYTE G_FAR *KeyValue)
*
* Description : This function loads two simple credit keys in the key file
*
* Remarks     : Nothing.
*
* In          : ChannelID:       The channel identifier.
*               KeyValue:        The value of the key (2 x 8 bytes)
*
* Out         : Nothing.

*****************************************************************************************/
void LoadCreditKey(INT16 ChannelID,BYTE G_FAR *KeyValue)
{
   INT16 
      response,  
      KeyNumber,                                               
      Mode,
      KeyType;
    
/*    Initialization of the variables                                                   */

   Mode       = NO_SELECT;  /* Does not use select file                                 */
   KeyNumber  = 2;          /* Number of the key in the key file                        */ 
   KeyType    = PAY;        /* Payment key                                              */  
 
    
   printf("* Load credit keys: ");
   
   response = G_MPC_LoadKey(ChannelID,
                            Mode,
                            0,
                            KeyNumber,
                            KeyType,  
                            0,
                            KeyValue                            
                            );
   
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_LoadKey returns %d.\n",response);
   }
   else{
      Mode       = NO_SELECT;  /* Does not use select file                                 */
      KeyNumber  = 3;          /* Number of the key in the key file                        */ 
      KeyType    = PAY;        /* Payment key                                              */  
    
      response = G_MPC_LoadKey(ChannelID,
                               Mode,
                               0,
                               KeyNumber,
                               KeyType,  
                               0,
                               KeyValue +8
                               );
      
      if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_LoadKey returns %d.\n",response);
      }
      else{
         printf("OK.\n");
      }
   }
}
/****************************************************************************************
*void CreatePurseEF(INT16 ChannelID,WORD16 PurseEFIdentifier)
*
* Description : This function creates a purse file
*
* Remarks     : Nothing.
*
* In          : ChannelID:         The channel identifier
*    .          PurseEFIdentifier: The purse file identifier
*
* Out         : Nothing.

*****************************************************************************************/
void CreatePurseEF(INT16 ChannelID,WORD16 PurseEFIdentifier)
{
   INT16 
      response,
      EF_Size;
   ACC
      SC_Group[3];  
   int i;
    
/*    Initialization of the variables                                                   */

   EF_Size = 20;    /* Size of th EF in bytes                                           */         
   
        
/*    Initialization of the access conditions                                           */
 
   for (i=0;i<3;i++){                /* No access conditions                            */
     SC_Group[i].Validity=0;
     SC_Group[i].KeyLevel=0;
     SC_Group[i].KeyShortID=0;
     SC_Group[i].Sc1Level=0;
     SC_Group[i].Sc2Level=0;
     SC_Group[i].Sc1Number=0;
     SC_Group[i].Sc2Number=0;
   }     
   printf("* Create purse EF: ");
    
   response = G_MPC_CreateEF(ChannelID, 
                             PurseEFIdentifier, 
                             EF_PURSE,
                             EF_Size,
                             &(SC_Group[0]),
                             &(SC_Group[1]),
                             &(SC_Group[2])
                             );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_CreateEF returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
}
/****************************************************************************************
* void SelectPurseEF(INT16 ChannelID, WORD16 PurseEFIdentifier)
*
* Description : This function selects a purse file 
*
* Remarks     : Nothing.
*
* In          : ChannelID:         The channel identifier.
*               PurseEFIdentifier: The purse file identifier.
*
* Out         : Nothing.

*****************************************************************************************/
void SelectPurseEF(INT16 ChannelID, WORD16 PurseEFIdentifier)
{
   WORD8 
      Descriptor[18];
   INT16 
      Mode,
      response;                        
    
   Mode = 2;              /* The mode is 1 to select a EF file                          */

   printf("* Select purse EF: ");
    
   response = G_MPC_Select(ChannelID, 
                           Mode, 
                           PurseEFIdentifier, 
                           Descriptor);

   /* Descriptor contains  "85" +                                                       */
   /*                       "10"(size of the DF descriptor in hexa) +                   */
   /*                        the 16 bytes of the DF descriptor +                        */

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_Select returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
}
/****************************************************************************************
*void LoadPurse(INT16 ChannelID,WORD16 PurseEFIdentifier,
*               INT32 MaxAllowedBalance,INT32 MaxFreeDebit,WORD16 KeyEFIdentifier)
*
* Description : This function initializes the parameters of a purse
*
* Remarks     : Nothing.
*
* In          : ChannelID:         The channel identifier.
*               PurseEFIdentifier: The purse file identifier
*                MaxAllowedBalance: The maximum amount allowed in a balance.     
*                MaxFreeDebit:      The maximum debit without the secret code  
*                KeyEFIdentifier:   The key file identifier
*
* Out         : Nothing.
*
*****************************************************************************************/
void LoadPurse(INT16 ChannelID,WORD16 PurseEFIdentifier,
               INT32 MaxAllowedBalance,INT32 MaxFreeDebit,WORD16 KeyEFIdentifier)
{
   INT16 
      response,
      DebitCodeNumber,
      ReadCodeNumber;
    
/*   Initialization of access conditons                                                 */

   DebitCodeNumber = 0;  /* No access condition for debit command                       */
   ReadCodeNumber  = 0;  /* No access condition for read balance command                */
                     
    
   printf("* Load purse: ");
    
    
   response = G_MPC_LoadPurseStandart(ChannelID, 
                                      PurseEFIdentifier,
                                      MaxAllowedBalance,
                                      KeyEFIdentifier,
                                      MaxFreeDebit,
                                      DebitCodeNumber,
                                      ReadCodeNumber
                                      );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_LoadPurseStandart returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
}
/****************************************************************************************
* void InitBalance(INT16 ChannelID,INT16 PurseEFIdentifier,INT32 BalanceAmount)
*
* Description : This function loads a balance amount in a purse file
*
* Remarks     : Nothing.
*
* In          : ChannelID:         The channel identifier.
*               PurseEFIdentifier: The purse file identifier
*               BalanceAmount:     The value of the balance to load.
*
* Out         : Nothing.

*****************************************************************************************/
void InitBalance(INT16 ChannelID,INT16 PurseEFIdentifier,INT32 BalanceAmount)
{
   INT16 
      response;
     
   printf("* InitBalance: ");
    
    
   response = G_MPC_InitBalance(ChannelID, 
                                PurseEFIdentifier,
                                BalanceAmount
                               );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_InitBalance returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
}
/****************************************************************************************
* BOOL CreateManagerEF(INT16 ChannelID,WORD16 TransManagerEFIdentifier)
*
* Description : This function creates an EF
*
* Remarks     : Nothing.
*
* In          : ChannelID:       The channel identifier
*               KeyEFIdentifier: The identifier of the EF to create
*
* Out         : Nothing.
*
* Response    : TRUE if the G_MPC_CreateEF function returns G_OK
*               FALSE if the G_MPC_CreateEF function returns an error code
*
*****************************************************************************************/
BOOL CreateManagerEF(INT16 ChannelID,WORD16 TransManagerEFIdentifier)
{

   INT16 
      response,
      EF_Size;
   ACC   
      SC_Group[3];  
   int i;
    
/*    Initialization of the variables                                                   */

   EF_Size = 16;    /* Size of th EF in bytes                                           */         
   
        
/*    Initialization of the access conditions                                           */
 
   for (i=0;i<3;i++){                /* No access conditions                            */
     SC_Group[i].Validity=0;
     SC_Group[i].KeyLevel=0;
     SC_Group[i].KeyShortID=0;
     SC_Group[i].Sc1Level=0;
     SC_Group[i].Sc2Level=0;
     SC_Group[i].Sc1Number=0;
     SC_Group[i].Sc2Number=0;
   }     
   printf("* Create Manager EF: ");
    
   response = G_MPC_CreateEF(ChannelID,
                             TransManagerEFIdentifier, 
                             EF_MANAGER, 
                             EF_Size, 
                             &SC_Group[0], 
                             &SC_Group[1], 
                             &SC_Group[2]);
    

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_CreateEF returns %d.\n",response);    
         return(FALSE);
   }
   else{
         printf("OK.\n");  
         return(TRUE);
   }
}
/****************************************************************************************
*void LoadManagerEF(INT16 ChannelID,WORD16 TransManagerEFIdentifier,INT32 CTC)
*
* Description : This function initializes the counters of the counter manager file 
*
* Remarks     : Nothing.
*
* In          : ChannelID:                The channel Identifier.
*               TransManagerEFIdentifier: The manager file identifier
*               CTC:                      The card counter manager
*
* Out         : Nothing.

*****************************************************************************************/
void LoadManagerEF(INT16 ChannelID,WORD16 TransManagerEFIdentifier,INT32 CTC)
{
   INT16 
      response;                                  
    
   printf("* Load Manager EF: ");
    
   response = G_MPC_LoadCounterManager(ChannelID,
                                       TransManagerEFIdentifier, 
                                       CTC, 
                                       0);
    
   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_LoadCounterManager returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
}

/****************************************************************************************
* void  main()
*
* Description : Main program.
*               Executes the different steps of a purse creation.
*
*
*****************************************************************************************/

void main()
{ 
   INT16 
      ChannelID;               /* Channel Identifier                                    */
   WORD16 
      DFIdentifier,            /* DF Identifier                                         */
      KeyEFIdentifier,         /* Key file identifier                                   */
      PurseEFIdentifier,       /* Purse file identifier                                 */
      SystemKeyEFIdentifier,   /* System key file identifier                            */
      TransManagerEFIdentifier;/* Transaction manager file identifier.                  */
   WORD8
      CreditKey[16],           /* Credit key                                            */
      PaymentKey[16],          /* Payment key                                           */
      DFName[16];              /* Name of the DF                                        */
   INT32
      MaxAllowedBalance,       /* The maximum amount allowed in a balance.              */ 
      MaxFreeDebit,            /* The maximum debit without the secret code             */
      BalanceAmount;           /* Initialization value of the balance.                  */
   WORD32 
      CTC;                     /* Card Transaction Counter                              */
   int i;

     
/* The following initializations are common to the 3 programs MPCPurse, MPCCred, MPCDeb */

   for (i=0;i<8;i++)  PaymentKey[i] = 0x12;   /* Initialization of the payment key      */
   for (i=8;i<16;i++) PaymentKey[i] = 0x56;   /* Initialization of the payment key      */
   for (i=0;i<8;i++)  CreditKey[i] = 0x34;    /* Initialization of the credit key       */
   for (i=8;i<16;i++) CreditKey[i] = 0x78;    /* Initialization of the credit key       */
 

/* Sample information text                                                              */

   printf("--------------------------------------------------------------------------\n");
   printf("MPCOS-3DES Library                                                        \n");
   printf("Reader: GCR 400 Serial/9600 on COM 1                                      \n");
   printf("--------------------------------------------------------------------------\n");
   printf("This sample shows the different steps in the creation of a standard purse.\n");
   printf("                                                                          \n");
   printf("The program asks you to enter a DF identifier. From this value it creates \n");
   printf("the other keys and the default values as follows.                         \n");
   printf("   Purse EF Identifier               =   DF Identifier + 1                \n");
   printf("   Key EF Identifier                 =   DF Identifier + 2                \n");
   printf("   Transaction Manager EF Identifier =   DF Identifier + 3                \n");
   printf("   MaxAllowedBalance                 =   50000                            \n");
   printf("   MaxFreeDebit                      =   10000                            \n");
   printf("   BalanceAmount                     =   15000                            \n");
   printf("   CTC                               =   0                                \n");
   printf("                                                                          \n");
   printf(" For each step of the purse creation, the sample displays the name of the \n");
   printf(" function used and the status returned.                                   \n");
   printf("                                                                          \n");
   printf("--------------------------------------------------------------------------\n");
   printf("                                                                          \n");
   printf(" press <ENTER> to run the sample.                                         \n");
   printf("                                                                          \n");

   getchar();
/* The value of the DF is the only parameter asked to the user                          */

   printf("Enter the DF identifier XXXX (hexa):"); 
   scanf("%04X",&DFIdentifier);
   
/* The bulding of the Purse file identifier and the key file identifier from the value  */
/* of the DF identifier is common to the 3 programs MPCPurse, MPCCred, MPCDeb           */

   PurseEFIdentifier        =   DFIdentifier + 1;
   KeyEFIdentifier          =   DFIdentifier + 2;  
   TransManagerEFIdentifier =   DFIdentifier + 3;
         
   printf(" DF Identifier                       %04X\n",DFIdentifier);
   printf(" Purse EF Identifier                 %04X\n",PurseEFIdentifier);
   printf(" Key EF Identifier                   %04X\n",KeyEFIdentifier);
   printf(" Transaction manager EF Identifier   %04X\n",TransManagerEFIdentifier);
        
/* Initialization of the variables used in this sample                                  */

   MaxAllowedBalance     = 50000;
   MaxFreeDebit          = 10000;
   BalanceAmount         = 15000;
   CTC                   = 0;
   SystemKeyEFIdentifier = KEY_FILE_ID;  
   strcpy (DFName,"MPCOSTEST");
         
   ChannelID = OpenChannel(IFD_TYPE);              /* Open channel                      */
   if (ChannelID >= G_OK) {
      
         OpenSession(ChannelID);                   /* Open session                      */
         
         SelectSystemKey(ChannelID,                /* Select system key                 */
                         SystemKeyEFIdentifier);  
         CreateDF(ChannelID,                       /* Create DF                         */
                  DFIdentifier,
                  DFName); 
         SelectDF(ChannelID,                       /* Select DF                         */
                  DFIdentifier);  
         CreateKeyEF(ChannelID,                    /* Create key file                   */
                     KeyEFIdentifier);
         LoadPayKey(ChannelID,                     /* Load payment key                  */
                    KeyEFIdentifier,
                    PaymentKey);
         LoadCreditKey(ChannelID,                  /* Load credit key                   */
                       CreditKey);
         CreatePurseEF(ChannelID,                  /* Create purse file                 */
                       PurseEFIdentifier);
         SelectPurseEF(ChannelID,                  /* Select purse file                 */
                       PurseEFIdentifier);
         LoadPurse(ChannelID,                      /* Load purse                        */
                   PurseEFIdentifier,
                   MaxAllowedBalance,
                   MaxFreeDebit,
                   KeyEFIdentifier);
         InitBalance(ChannelID,                    /* Init balance                      */
                     PurseEFIdentifier,
                     BalanceAmount);
         if(CreateManagerEF(ChannelID,             /* Create manager file               */
                            TransManagerEFIdentifier)
                            == TRUE)
         LoadManagerEF(ChannelID,                  /* Load manager file                 */
                       TransManagerEFIdentifier,
                       CTC);
         CloseSession(ChannelID);                  /* Close session                     */
         CloseChannel(ChannelID);                  /* Close channel                     */
   }

}